/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2008 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.proxies;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.bf.BindHints;

/**
 * A stub component factory can create components that behave as stubs for other
 * components
 */
public interface StubComponentFactory {
	/**
	 * Create a stub component for the specified {@link InterfaceType}.
	 * 
	 * @param iType
	 *            the {@link InterfaceType} to be used to generate the
	 *            InterfaceType of the stub
	 * @param itfName
	 *            the name of the client interface the stub will be bound to
	 * @return the stub {@link Component}
	 * @throws InstantiationException
	 *             if some error occurs while instantiating the component
	 * @see org.objectweb.fractal.api.type.InterfaceType
	 */
	public Component createStubComponent(InterfaceType iType,
			BindHints bindHints) throws InstantiationException;
}
